using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using TMServerAccess;

namespace ServerInterfaceCSharp
{
	public partial class Form1 : Form
	{
		public TMServerAccess.Telephony m_Telephony;
		public TMServerAccess.ClientCom m_ClientCom;
		public TMServerAccess.CtiConvert m_CtiConvert;
		public TMServerAccess.PBXInfo m_PBXInfo;

		public Form1()
		{
			InitializeComponent();

			m_Telephony = new TMServerAccess.Telephony();
			m_Telephony.OnCallNumber += OnCallNumber;
			m_Telephony.OnCallState += OnCallState;
			m_Telephony.OnIdentity += OnIdentity;
			m_Telephony.OnNewCall += OnNewCall;
			m_Telephony.OnProtocolData += OnProtocolData;

			m_ClientCom = new TMServerAccess.ClientCom();
			m_ClientCom.OnUserToDatabase += OnUserToDatabase;

			m_CtiConvert = new TMServerAccess.CtiConvert();

			m_PBXInfo = new TMServerAccess.PBXInfo();

			m_Telephony.Init();
		}

		// Init CTI interface
		private void Form1_Load(object sender, EventArgs e)
		{
			label18.Text = Convert.ToString(m_PBXInfo.LicenseCount);
		}

		private void button1_Click(object sender, EventArgs e)
		{
			String szFormattedNumber = "";
			szFormattedNumber = m_CtiConvert.ConvertToCanonical(textBox1.Text);
			textBox2.Text = szFormattedNumber;
		}

		// Make a call
		private void button2_Click(object sender, EventArgs e)
		{
			m_Telephony.MakeCall(textBox3.Text, textBox4.Text);
		}

		// Answer a call
		private void button3_Click(object sender, EventArgs e)
		{
			m_Telephony.Answer(textBox3.Text);
		}

		// Drop a call
		private void button4_Click(object sender, EventArgs e)
		{
			m_Telephony.Hangup(textBox3.Text);
		}

		// Set display text of a phone device (if supported)
		private void button5_Click(object sender, EventArgs e)
		{
			m_Telephony.SetPhoneDisplay(textBox5.Text, textBox6.Text);
		}

		// Send identity data to the client
		private void button6_Click(object sender, EventArgs e)
		{
			UInt32 dwCallID = Convert.ToUInt32(textBox8.Text);
			m_ClientCom.SendIdentity(dwCallID, textBox7.Text, textBox9.Text);
		}

		// Send user defined data to the client
		private void button7_Click(object sender, EventArgs e)
		{
			UInt32 dwCommand = Convert.ToUInt32(textBox11.Text);
			m_ClientCom.SendUserData(dwCommand, textBox12.Text, textBox10.Text);
		}

		// Clear event list
		private void button8_Click(object sender, EventArgs e)
		{
			listBox1.Items.Clear();
		}

		// Close form
		private void button9_Click(object sender, EventArgs e)
		{
			Close();
		}

		private void button10_Click(object sender, EventArgs e)
		{
			m_Telephony.Redirect(textBox3.Text, textBox4.Text);
		}

		private void button11_Click(object sender, EventArgs e)
		{
			m_Telephony.Alternate(textBox3.Text);
		}

		private void button12_Click(object sender, EventArgs e)
		{
			m_Telephony.Transfer(textBox3.Text);
		}

		private void button13_Click(object sender, EventArgs e)
		{
			m_Telephony.Conference(textBox3.Text);
		}

		private void button14_Click(object sender, EventArgs e)
		{
			m_Telephony.Pickup(textBox3.Text, textBox4.Text);
		}

		public void AddListBoxText(string strText)
		{
			listBox1.Items.Add(strText);
		}

		private void OnCallNumber(object sender, TMServerAccessArgs.CallNumberArgs e)
		{
			String szCallID = Convert.ToString(e.dwCallID);
			AddListBoxText("CTI_CALLNUMBER");
			AddListBoxText("   Internal: " + e.strDevice);
			AddListBoxText("   Call: " + szCallID);
			AddListBoxText("   Number: " + e.strNumber);
		}

		private void OnCallState(object sender, TMServerAccessArgs.CallStateArgs e)
		{
			String szCallID = Convert.ToString(e.dwCallID);
			String szState = Convert.ToString(e.dwState);
			AddListBoxText("CTI_CALLSTATE");
			AddListBoxText("   Internal: " + e.strDevice);
			AddListBoxText("   Call: " + szCallID);
			AddListBoxText("   State: " + szState);
		}

		private void OnIdentity(object sender, TMServerAccessArgs.IdentityArgs e)
		{
			String szCallID = Convert.ToString(e.dwCallID);
			AddListBoxText("CTI_GETIDENTITY");
			AddListBoxText("   Internal: " + e.strDevice);
			AddListBoxText("   Call: " + szCallID);
			AddListBoxText("   Number: " + e.strNumber);
		}

		private void OnNewCall(object sender, TMServerAccessArgs.NewCallArgs e)
		{
			String szCallID = Convert.ToString(e.dwCallID);
			String szInbound = "No";
			if (e.bInbound == true) { szInbound = "Yes"; }
			String szStartTime = "";
			szStartTime = m_CtiConvert.GetDateTime(e.dwStart);
			AddListBoxText("CTI_NEWCALL");
			AddListBoxText("   Internal: " + e.strDevice);
			AddListBoxText("   Call: " + szCallID);
			AddListBoxText("   Inbound: " + szInbound);
			AddListBoxText("   Time: " + szStartTime);
			AddListBoxText("   Partner: " + e.strNumber);
		}

		private void OnProtocolData(object sender, TMServerAccessArgs.ProtocolDataArgs e)
		{
			String szConnected = "No";
			if (e.bSuccess == true) { szConnected = "Yes"; }
			String szInbound = "No";
			if (e.bInbound == true) { szInbound = "Yes"; }
			String szStartTime = "";
			String szEndTime = "";
			szStartTime = m_CtiConvert.GetFormatDateTime(e.dwStart, "%H:%M:%S");
			szEndTime = m_CtiConvert.GetFormatDateTime(e.dwEnd, "%H:%M:%S");
			AddListBoxText("CTI_PROTOCOLDATA");
			AddListBoxText("   Internal: " + e.strDevice);
			AddListBoxText("   Partner: " + e.strNumber);
			AddListBoxText("   Address: " + e.strAddress);
			AddListBoxText("   Connected: " + szConnected);
			AddListBoxText("   Inbound: " + szInbound);
			AddListBoxText("   StartTime: " + szStartTime);
			AddListBoxText("   EndTime: " + szEndTime);
		}

		private void OnUserToDatabase(object sender, TMServerAccessArgs.UserToDatabaseArgs e)
		{
			String szCommand = Convert.ToString(e.dwCommand);
			AddListBoxText("CTI_USERTODATABASE");
			AddListBoxText("   Internal:" + e.strDevice);
			AddListBoxText("   Command:" + szCommand);
			AddListBoxText("   Text:" + e.strMessage);
		}
	}
}