//****************************************************************************************
// CTI-SDK: Public data types and constants
// Copyright  2001 - 2024 Tino Kasubke
//****************************************************************************************
#ifndef __CTITYPES_H__
#define __CTITYPES_H__
//***************************************************************************************
#include <time.h>
#include <tapi.h>
//***************************************************************************************
// Export definitions
//***************************************************************************************
#ifndef DLL_API
#ifdef DLL_EXPORTS
#define DLL_API extern "C" __declspec(dllexport)
#else
#define DLL_API extern "C" __declspec(dllimport)
#endif
#endif

#ifndef DLL_API2
#ifdef DLL_EXPORTS
#define DLL_API2 __declspec(dllexport)
#else
#define DLL_API2 __declspec(dllimport)
#endif
#endif
//****************************************************************************************
// General definitions
//****************************************************************************************
// For unicode
#ifndef TCHAR_SIZE
#define TCHAR_SIZE sizeof(TCHAR)
#endif

// 64 Bit word
#ifndef QWORD
typedef unsigned __int64 QWORD, FAR *PQWORD;
#endif

// String overlay type for internal direct diallings
#ifdef _UNICODE
typedef QWORD CTI_NUMBER, FAR *LP_CTI_NUMBER;
typedef WPARAM XWPARAM,FAR* LPXWPARAM;
typedef LPARAM XLPARAM,FAR* LPXLPARAM;
#else
typedef DWORD CTI_NUMBER, FAR *LP_CTI_NUMBER;
typedef DWORD XWPARAM,FAR* LPXWPARAM;
typedef LONG XLPARAM,FAR* LPXLPARAM;
#endif

// Fixed string size
#define STD_INTERN_LENGTH    2
#define INTERN_NUMBER_SIZE   4
#define PORT_NUMBER_SIZE     5
#define CTI_NUMBER_SIZE      8
#ifndef PHONE_NUMBER_SIZE
#define PHONE_NUMBER_SIZE   32
#endif
#define EDV_NAME_SIZE       32
#define ICON_TOOLTIP_SIZE   64
#define LINE_NAME_SIZE      64
#define DBBOOK_NAME_SIZE    64
#define DBBOOK_EXTRA_SIZE  128
#define TITLE_SIZE         128
#define CTI_MESSAGE_SIZE   256
#define SQLSTRING_SIZE    1024

// Information types for incoming calls
enum USER_CALL_INFO
{
   UCI_NONE       = 0,// Do nothing
   UCI_SHOWCLIENT = 1,// Display client
   UCI_CUSTOM     = 2,// Display using cusom functions
};
//***************************************************************************************
// Structures for send- ans receive buffers
//***************************************************************************************
#pragma pack(1)

// Help structure for lineGetID
typedef struct varstring_dword_tag
{
   DWORD dwTotalSize;
   DWORD dwNeededSize;
   DWORD dwUsedSize;
   DWORD dwStringFormat;
   DWORD dwStringSize;
   DWORD dwStringOffset;
   DWORD dwLineId;
   BYTE bBuffer[64];
}WORD_VARSTRING,FAR *LPWORD_VARSTRING;
#pragma pack()
//****************************************************************************************
// Service error code
//****************************************************************************************
const DWORD FCTISRV_ERROR_NOMEMORY        = 0x00000001;// Speicherreservierung schlug fehl
const DWORD FCTISRV_ERROR_NOPROCESSCOMM   = 0x00000002;// Kein Prozekommunikation mglich
const DWORD FCTISRV_ERROR_NOTAPIMANAGER   = 0x00000003;// Tapi-Manager nicht geestartet
const DWORD FCTISRV_ERROR_NOSERVICEHANDLE = 0x00000004;// Dienstesteuerung nicht ansprechbar
const DWORD FCTISRV_ERROR_NONETWORK       = 0x00000005;// Netzwerk nicht verfgbar
//****************************************************************************************
// Netzwerk-Befehlsstrukturen
//****************************************************************************************
#pragma pack(1)
//***************************************************************************************

// Help structure for translation from COMPARE_NUMBER to TCHAR[]
typedef union cti_number_map
{
   CTI_NUMBER wNumber;
   TCHAR szNumber[CTI_NUMBER_SIZE];
}CTI_NUMBER_MAP,LP_CTI_NUMBER_MAP;

typedef union cti_number_map_a
{
   DWORD wNumber;
   CHAR szNumber[CTI_NUMBER_SIZE];
}CTI_NUMBER_MAP_A,LP_CTI_NUMBER_MAP_A;

// Network command structure
typedef struct cti_command_tag
{
   DWORD m_dwSize;
   DWORD m_dwCommand;
   XWPARAM m_wParam;
   XLPARAM m_lParam;
}CTI_COMMAND,FAR* LP_CTI_COMMAND;
const DWORD CTI_COMMAND_SIZE = sizeof(CTI_COMMAND);

// The same command with the direct dial
typedef struct cti_command_tag_ex
{
   DWORD m_dwSize;
   DWORD m_dwCommand;
   XWPARAM m_wParam;
   XLPARAM m_lParam;
#ifdef USE_ANSI_STRUCTS
   CTI_NUMBER_MAP_A m_unNumber;
#else
   CTI_NUMBER_MAP m_unNumber;
#endif
}CTI_COMMAND_EX,FAR* LP_CTI_COMMAND_EX;
const DWORD CTI_COMMAND_EX_SIZE = sizeof(CTI_COMMAND_EX);

// The same command with an additional string
typedef struct cti_command_tag_msg
{
   DWORD m_dwSize;
   DWORD m_dwCommand;
   XWPARAM m_wParam;
   XLPARAM m_lParam;
#ifdef USE_ANSI_STRUCTS
   CTI_NUMBER_MAP_A m_unNumber;
   CTI_NUMBER_MAP_A m_unOther;
   CHAR m_szMessage[CTI_MESSAGE_SIZE];
#else
   CTI_NUMBER_MAP m_unNumber;
   CTI_NUMBER_MAP m_unOther;
   TCHAR m_szMessage[CTI_MESSAGE_SIZE];
#endif
}CTI_COMMAND_MSG,FAR* LP_CTI_COMMAND_MSG;
const DWORD CTI_COMMAND_MSG_SIZE = sizeof(CTI_COMMAND_MSG);// 288 bei ANSI
//****************************************************************************************
// Call data record
//****************************************************************************************
// The data record and the network commands are header-compatible
typedef struct cti_phone_data_tag
{
   // Base structure
   DWORD m_dwSize;
   DWORD m_dwCommand;
   __time32_t m_dwStart;
   __time32_t m_dwEnd;
   DWORD m_dwFlags;
#ifdef USE_ANSI_STRUCTS
   CTI_NUMBER_MAP_A m_unActor;
   CHAR m_szNumber[PHONE_NUMBER_SIZE];
#else
   CTI_NUMBER_MAP m_unActor;
   TCHAR m_szNumber[PHONE_NUMBER_SIZE];
#endif
   // Additional reminder info
#ifdef USE_ANSI_STRUCTS
   CTI_NUMBER_MAP_A m_unDest;
   CHAR m_szMessage[CTI_MESSAGE_SIZE];
#else
   CTI_NUMBER_MAP m_unDest;
   TCHAR m_szMessage[CTI_MESSAGE_SIZE];
#endif
}CTI_PHONE_DATA,FAR* LP_CTI_PHONE_DATA;
const DWORD CTI_PHONE_DATA_SIZE = sizeof(CTI_PHONE_DATA);

const DWORD CTI_PHONE_DATA_FIX_SIZE = 
	sizeof(DWORD) * 3 + 
	sizeof(__time32_t) * 2 +
	sizeof(CTI_NUMBER_MAP) +
	(PHONE_NUMBER_SIZE * TCHAR_SIZE);

const DWORD CTI_PHONE_DATA_RECALL_FIX_SIZE = CTI_PHONE_DATA_FIX_SIZE
 + (CTI_NUMBER_SIZE * TCHAR_SIZE);
//***************************************************************************************
// TSP control
//***************************************************************************************
typedef struct cti_tsp_data
{
   DWORD m_dwCommand;
	HCALL m_hCall;
   LONG m_lParam;
   CHAR m_szNumber[PHONE_NUMBER_SIZE];
}CTI_TSP_DATA,FAR* LP_CTI_TSP_DATA;
//***************************************************************************************
#pragma pack()
//****************************************************************************************
// Network command identifiers
//****************************************************************************************
// Control commands
const DWORD MASK_CTI_COMMAND     = 0x00010000;// Mask

// Network commands
const DWORD CTI_LOGIN            = 0x00011001;
const DWORD CTI_LOGOFF           = 0x00011002;

// Telephony
const DWORD CTI_CALLNUMBER       = 0x00012001;
const DWORD CTI_MAKECALL         = 0x00012002;
const DWORD CTI_DROPCALL         = 0x00012003;
const DWORD CTI_FORWARDINGSTATE  = 0x00012004;
const DWORD CTI_OBJECTOFREDIRECT = 0x00012006;
const DWORD CTI_ANSWERCALL       = 0x00012007;
const DWORD CTI_HOLD             = 0x00012008;
const DWORD CTI_UNHOLD           = 0x00012009;
const DWORD CTI_REDIRECTCALL     = 0x0001200A;
const DWORD CTI_PROTOCOLDATA     = 0x0001200B;
const DWORD CTI_CALLREMINDER     = 0x0001200C;
const DWORD CTI_IDENTITY         = 0x0001200D;
const DWORD CTI_PICKUP           = 0x0001200E;
const DWORD CTI_CALLCHANGE       = 0x0001200F;
const DWORD CTI_TRANSFER         = 0x00012010;
const DWORD CTI_SENDUSERSTATE    = 0x00012011;
const DWORD CTI_ADDCALLTOCONF    = 0x00012012;
const DWORD CTI_ADDNUMBERTOCONF  = 0x00012013;
const DWORD CTI_CONFNOTIFY       = 0x00012014;
const DWORD CTI_LINERESET        = 0x00012015;
const DWORD CTI_SENDADDRESS      = 0x00012016;
const DWORD CTI_PROXYMAKECALL	 = 0x00012017;
const DWORD CTI_NEWCALL			 = 0x00012018;
const DWORD CTI_LOGINDATA		 = 0x00012019;
const DWORD CTI_TRANSFERPENDINFO = 0x0001201A;
const DWORD CTI_FORWARD          = 0x0001201B;

// Additional commands
const DWORD CTI_SERVICEPAUSE     = 0x00013001;
const DWORD CTI_CLIENTCLOSE      = 0x00013002;
const DWORD CTI_SERVERMESSAGE    = 0x00013003;
const DWORD CTI_WRONGCHARSIZE    = 0x00013004;
const DWORD CTI_USERTOUSER       = 0x00013005;
const DWORD CTI_CHANGESERVER     = 0x00013006;
const DWORD CTI_DATABASETOUSER   = 0x00013007;// Duplicate, direction dependend
const DWORD CTI_USERTODATABASE   = 0x00013007;// Duplicate, direction dependend
const DWORD CTI_GROUPMEMBER      = 0x00013008;
const DWORD CTI_TESTMESSAGE      = 0x00013009;
const DWORD CTI_CHATSTART        = 0x0001300A;
const DWORD CTI_CHATSTOP         = 0x0001300B;
const DWORD CTI_CHATMESSAGE      = 0x0001300C;
const DWORD CTI_LOCALEINFO       = 0x0001300D;
const DWORD CTI_LICENSEINFO      = 0x0001300E;
const DWORD CTI_ACDCANCELCLIENT	 = 0x0001300F;
const DWORD CTI_REBARSIZECHANGED = 0x00013010;

// Internal commands
const DWORD CTI_CPLTRANSFER      = 0x0001F001;
const DWORD CTI_PERMISSIONCHANGE = 0x0001F002;
const DWORD CTI_STARTTEST        = 0x0001F003;
const DWORD CTI_ENDTEST          = 0x0001F004;
const DWORD CTI_TESTBREAKTIME    = 0x0001F005;
const DWORD CTI_LINEFEATURES     = 0x0001F006;
const DWORD CTI_ACDREADY		 = 0x0001F00A;
const DWORD CTI_SETPHONEDISPLAY	 = 0x0001F00F;
const DWORD CTI_SETTAPINUMBER	 = 0x0001F010;
const DWORD CTI_TAPISPECIAL      = 0x0001F011;
const DWORD CTI_TESTSTATE        = 0x0001F012;
const DWORD CTI_TESTFEATURES     = 0x0001F013;
const DWORD CTI_TESTRESET        = 0x0001F014;
const DWORD CTI_TESTRESULTS1     = 0x0001F015;
const DWORD CTI_TESTRESULTS2     = 0x0001F016;
const DWORD CTI_TESTRESULTS3     = 0x0001F017;
const DWORD CTI_TESTRESULTS4     = 0x0001F018;
const DWORD CTI_TESTRESULTS5     = 0x0001F019;
const DWORD CTI_REOPENNAMEDPIPE  = 0x0001F01A;
const DWORD CTI_LOGINCLIENTS		= 0x0001F01D;
const DWORD CTI_USERCALLSTATES	= 0x0001F01E;
const DWORD CTI_DATABASESETTINGS	= 0x0001F01F;

// Diagnosis
const DWORD MASK_CTI_DIAGNOSE    = 0x00020000;// Mask

// TAPI commands
const DWORD MASK_CTI_TAPI        = 0x00030000;// Mask
const DWORD CTI_CALLINFO         = 0x00030001;
const DWORD CTI_CALLSTATE        = 0x00030002;
const DWORD CTI_GATHERDIGITS     = 0x00030003;

// SQL commands
const DWORD MASK_SQL             = 0x00040000;
const DWORD SQL_LOGIN            = 0x00040001;
const DWORD SQL_DATA             = 0x00040002;
const DWORD SQL_NEXT             = 0x00040003;
const DWORD SQL_END              = 0x00040004;
const DWORD SQL_DBBOOK           = 0x00040005;
const DWORD SQL_INSERTDATAREC    = 0x00040006;
const DWORD SQL_DELETEDATAREC    = 0x00040007;
//****************************************************************************************
// Window commands, also used in callback functions. The interval from WM_USER + 1 up to
// WM_USER + 100 is in use.
//****************************************************************************************
// Network
const DWORD WM_CTI_NETWORK_DISPATCH       = WM_USER +  21;// Distribute messages
const DWORD WM_CTI_NETWORKOPEN            = WM_USER +  22;// Network connected
const DWORD WM_CTI_NETWORKCLOSE           = WM_USER +  23;// Network interrupted
// Tray bar icons
const DWORD WM_CTI_MENUCOMMAND            = WM_USER +  31;// Menu command from connection icon
const DWORD WM_CTI_SHOWPHONELIST          = WM_USER +  32;// Phone icon
const DWORD WM_CTI_RAISECLIENT            = WM_USER +  33;// Show client in the foreground
// Internal commands
const DWORD WM_CONNECT_CTI_BROWSER        = WM_USER +  41;// 
const DWORD WM_SETALWAYS_ON_TOP           = WM_USER +  42;// 
const DWORD WM_SHOWPRIVATE_LIST           = WM_USER +  51;// 
const DWORD WM_SHOWGROUP_LIST             = WM_USER +  52;// 
const DWORD WM_ACTION_PRIVATE_LIST        = WM_USER +  53;// 
const DWORD WM_ACTION_GROUP_LIST          = WM_USER +  54;// 
const DWORD WM_SECONDIDENTITIY            = WM_USER +  55;// 
const DWORD WM_OUTLOOKREADY               = WM_USER +  56;// 
const DWORD WM_CALLINFOENABLE             = WM_USER +  57;// 
// Database
const DWORD WM_CTI_DATABASE_MESSAGE       = WM_USER +  61;// Database messages
const DWORD WM_CTI_DATABASE_MESSAGEEX     = WM_USER +  62;// Database messages
//****************************************************************************************
// Client flags
//****************************************************************************************
// Network
const DWORD CTICLIENT_AUTOCONNECT       = 0x00000002;// Keep connection up automatically
const DWORD CTICLIENT_SHOWICON          = 0x00000004;// Display connection icon
// Logon
const DWORD CTICLIENT_CHECKCONNECTION   = 0x00000008;// Check connection settings when starting
const DWORD CTICLIENT_AUTOLOGIN         = 0x00000010;// Automatic log-on
// Features
const DWORD CTICLIENT_SHOWMENU          = 0x00000020;// Tray icon has a menu
const DWORD CTICLIENT_HOTKEY            = 0x00000040;// Text-dial using keyboard short-cut
const DWORD CTICLIENT_PHONEICON         = 0x00000080;// Call symbol
const DWORD CTICLIENT_ASSISTEDTELEPHONY = 0x00000100;// Client supports Assisted Telephony
const DWORD CTICLIENT_AUTOANSWER        = 0x00000200;// Answer incoming calls automatically

// Other
const DWORD CTICLIENT_DEFAULT 
   = CTICLIENT_AUTOCONNECT |
   CTICLIENT_SHOWICON |
   CTICLIENT_CHECKCONNECTION |
   CTICLIENT_AUTOLOGIN |
   CTICLIENT_PHONEICON |
   CTICLIENT_ASSISTEDTELEPHONY;             // Default 0x0000019E
const DWORD CTICLIENT_ALLFLAGS = 0xFFFFFFFF;// All settings
//****************************************************************************************
#endif
//****************************************************************************************
