//****************************************************************************************
#include "stdafx.h"
//****************************************************************************************
#ifdef _DEBUG
#define new DEBUG_NEW
#endif
//****************************************************************************************
CMiniCTIInCDlg* CMiniCTIInCDlg::m_pThis = NULL;
//****************************************************************************************
// Constructor
CMiniCTIInCDlg::CMiniCTIInCDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CMiniCTIInCDlg::IDD, pParent)
{
	m_pThis = this;
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_bIsConnected = FALSE;
	m_bInitReady = FALSE;
	m_bActiveCall = FALSE;
	m_bRingingCall = FALSE;
	m_hIconOn = LoadIcon(GetModuleHandle(NULL),MAKEINTRESOURCE(IDI_ICONON));
	m_hIconOff = LoadIcon(GetModuleHandle(NULL),MAKEINTRESOURCE(IDI_ICONOFF));
	m_hMainMenu = LoadMenu(GetModuleHandle(NULL),MAKEINTRESOURCE(IDR_MENU1));
	m_hPopupMenu = GetSubMenu(m_hMainMenu,0);
	m_bIsActive = TRUE;
}
//****************************************************************************************
// Destructor
CMiniCTIInCDlg::~CMiniCTIInCDlg(VOID)
{
	m_bIsActive = FALSE;
	CTI_NClientStop();
	DestroyIcon(m_hIconOn);
	DestroyIcon(m_hIconOff);
	DestroyMenu(m_hPopupMenu);
	DestroyMenu(m_hMainMenu);
}
//****************************************************************************************
// Read dialog data
void CMiniCTIInCDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_NETWORK, m_Network);
	DDX_Control(pDX, IDC_ANSWER, m_Answer);
	DDX_Control(pDX, IDC_DROP, m_Drop);
	DDX_Control(pDX, IDC_BOOK, m_Book);
	DDX_Control(pDX, IDC_LIST, m_List);
}
//****************************************************************************************
BEGIN_MESSAGE_MAP(CMiniCTIInCDlg, CDialog)
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_NETWORK, OnBnClickedNetwork)
	ON_BN_CLICKED(IDC_ANSWER, OnBnClickedAnswer)
	ON_BN_CLICKED(IDC_DROP, OnBnClickedDrop)
	ON_BN_CLICKED(IDC_BOOK, OnBnClickedBook)
	ON_BN_CLICKED(IDC_LIST, OnBnClickedList)
	ON_WM_MOVE()
END_MESSAGE_MAP()
//****************************************************************************************
// Create dialog
BOOL CMiniCTIInCDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	SetIcon(m_hIcon, TRUE);	
	SetIcon(m_hIcon, FALSE);

	// Create subdialogs
	g_BookDlg.Create(IDD_BOOKDLG,this);
	g_ListDlg.Create(IDD_LISTDLG,this);

	// Set window handle
	CTI_ESetMainWindow(m_hWnd);

	// Connection options
	CTI_OSetOptions(					
		CTICLIENT_AUTOCONNECT			// Keep connection up automatically
		| CTICLIENT_SHOWICON				// Display connection icon
		| CTICLIENT_CHECKCONNECTION	// Check connection settings when starting
		| CTICLIENT_AUTOLOGIN			// Automatic log-on
		| CTICLIENT_SHOWMENU				// Tray icon has a menu
		| CTICLIENT_PHONEICON,			// Call symbol
		0);									
	
	// Connection icon
	CTI_OConnectionIconOptions(m_hIconOn,m_hIconOff,m_hPopupMenu,_T("CTI demo client"));
	
	// Start network
	CTI_NStartClientCALLBACK(CTICallbackFunc);

	// Modify controls
	ActivateItems();
	
	m_bInitReady = TRUE;
	return TRUE;
}
//****************************************************************************************

// Activate dialog controls
VOID CMiniCTIInCDlg::ActivateItems(VOID)
{
	if(m_bIsActive)
	{
		m_Answer.EnableWindow(m_bIsConnected && m_bRingingCall);
		m_Drop.EnableWindow(m_bIsConnected && m_bActiveCall);
	}// if(m_bIsActive)
}
//****************************************************************************************
// Translate messages from CTI server
VOID CMiniCTIInCDlg::TranslateServerMessage(LP_CTI_COMMAND_MSG lpComMsg)
{
	TCHAR szBuffer[256] = _T("");
	
	// Evaluate phone messages
	switch(lpComMsg->m_dwCommand)
	{
	// Phone number
	case CTI_CALLNUMBER:
		{
			// Only incoming calls
			if(HIWORD(lpComMsg->m_wParam))
			{
				// Caller identification
				*szBuffer = 0;
				StringCchPrintf(szBuffer,256,g_BookDlg.GetIdentity(lpComMsg->m_szMessage));

				// Close phone book
				g_BookDlg.ShowWindow(SW_HIDE);
				m_Book.SetCheck(BST_UNCHECKED);

				// Open caller list
				g_ListDlg.ShowWindow(SW_SHOW);
				m_List.SetCheck(BST_CHECKED);
				
				// Create new item in caller list
				g_ListDlg.InsertCaller(lpComMsg->m_szMessage,szBuffer);
			}// if(HIWORD(lpComMsg->m_wParam))
			break;
		}// case CTI_CALLNUMBER
	// Call state
	case CTI_CALLSTATE:
		{
			// Read call state
			switch(lpComMsg->m_wParam)
			{
			case 0:
			case LINECALLSTATE_IDLE:
			case LINECALLSTATE_DISCONNECTED:
				{
					m_bActiveCall = FALSE;
					m_bRingingCall = FALSE;
					break;
				}
			case LINECALLSTATE_OFFERING:
			case LINECALLSTATE_ACCEPTED:
				{
					m_bActiveCall = TRUE;
					m_bRingingCall = TRUE;
					break;
				}
			default:
				{
					m_bRingingCall = FALSE;
					m_bActiveCall = TRUE;
					break;
				}
			}// switch(lpComMsg->m_wParam)m_bActiveCall

			// Update button controls
			ActivateItems();
			break;
		}// case CTI_CALLSTATE
	// Restart program
	case CTI_CLIENTCLOSE:
		{
			CTI_NPrepareRestart((LONG)lpComMsg->m_lParam);
			EndDialog(IDCANCEL);
			break;
		}// case CTI_CLIENTCLOSE
	}// switch(_lpComMsg->m_dwCommand)
}
//****************************************************************************************

// Callcack function
VOID WINAPI CMiniCTIInCDlg::CTICallbackFunc(DWORD dwCommand,WPARAM wParam,LPARAM lParam)
{
	switch(dwCommand)
	{
	// Menu commands from connection icon
	case WM_CTI_MENUCOMMAND:
		{
			switch(LOWORD(wParam))
			{
			case ID_1_ANRUFERLISTE:
				{
					m_pThis->m_List.SetCheck(BST_CHECKED);
					m_pThis->OnBnClickedList();
					break;
				}// case ID_1_ANRUFERLISTE
			case ID_1_TELEFONBUCH:
				{
					m_pThis->m_Book.SetCheck(BST_CHECKED);
					m_pThis->OnBnClickedBook();
					break;
				}// case ID_1_TELEFONBUCH
			case ID_1_BEENDEN:
				{
					m_pThis->EndDialog(IDCANCEL);
					break;
				}// case ID_1_BEENDEN
			}// switch(LOWORD(wParam))
			break;
		}// case WM_CTI_MENUCOMMAND
	// Display phone list
	case WM_CTI_SHOWPHONELIST:
		{
			m_pThis->m_List.SetCheck(BST_CHECKED);
			m_pThis->OnBnClickedList();
			break;
		}// case WM_CTI_SHOWPHONELIST
	// Network OK
	case WM_CTI_NETWORKOPEN:
		{
			m_pThis->m_bIsConnected = TRUE;
			m_pThis->ActivateItems();
			break;
		}// case WM_CTI_NETWORKOPEN
	// Network interrupted
	case WM_CTI_NETWORKCLOSE:
		{
			m_pThis->m_bIsConnected = FALSE;
			m_pThis->ActivateItems();
			break;
		}// case WM_CTI_NETWORKCLOSE
	// Messages from server
	case WM_CTI_NETWORK_DISPATCH:
		{
			m_pThis->TranslateServerMessage((LP_CTI_COMMAND_MSG)lParam);
			break;
		}// case WM_CTI_NETWORK_DISPATCH
	}// switch(pMsg->message)
}
//****************************************************************************************

// Change network settings
void CMiniCTIInCDlg::OnBnClickedNetwork()
{
	// Open network dialog
	CTI_NNetworkConfigDialog();
}
//****************************************************************************************

// Answer call
void CMiniCTIInCDlg::OnBnClickedAnswer()
{
	// Answers the first inbound call
	CTI_TCallAnswer(0);
}
//****************************************************************************************

// Hang up a call
void CMiniCTIInCDlg::OnBnClickedDrop()
{
	// Hang up the oldest call
	CTI_TCallDrop(0);
}
//****************************************************************************************

// Pull down phone book
void CMiniCTIInCDlg::OnBnClickedBook()
{
	if(m_Book.GetCheck())
	{
		// Close caller list
		m_List.SetCheck(BST_UNCHECKED);
		g_ListDlg.ShowWindow(SW_HIDE);

		// Open phone book list
		g_BookDlg.ShowWindow(SW_SHOW);
	}
	else
	{
		// Close phone book
		g_BookDlg.ShowWindow(SW_HIDE);
	}// if(m_Book.GetCheck())
}
//****************************************************************************************

// Pull down caller list
void CMiniCTIInCDlg::OnBnClickedList()
{
	if(m_List.GetCheck())
	{
		// Close phone book list
		m_Book.SetCheck(BST_UNCHECKED);
		g_BookDlg.ShowWindow(SW_HIDE);

		// Open caller list
		g_ListDlg.ShowWindow(SW_SHOW);
	}
	else
	{
		// Close caller list
		g_ListDlg.ShowWindow(SW_HIDE);
	}// if(m_List.GetCheck())
}
//****************************************************************************************
// Dialog moved
void CMiniCTIInCDlg::OnMove(int x, int y)
{
	CDialog::OnMove(x, y);

	// Change position of subdialogs
	if(m_bInitReady)
	{
		CRect Pos;
		GetWindowRect(&Pos);
		g_BookDlg.MoveWindow(Pos.left,Pos.bottom,Pos.right - Pos.left,LIST_SIZE);
		g_ListDlg.MoveWindow(Pos.left,Pos.bottom,Pos.right - Pos.left,LIST_SIZE);
	}// if(m_bInitReady)
}
//****************************************************************************************
