//****************************************************************************************
#include "stdafx.h"
//****************************************************************************************
// CBookDlg dialog
IMPLEMENT_DYNAMIC(CBookDlg, CDialog)
//****************************************************************************************
CBookDlg* CBookDlg::m_pThis = NULL;
//****************************************************************************************
// Construction
CBookDlg::CBookDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CBookDlg::IDD, pParent)
{
	m_pThis = this;
}
//****************************************************************************************
// Delivers the name of the caller
PTCHAR CBookDlg::GetIdentity(PTCHAR pszNumber)
{
	static TCHAR szReturn[64] = _T("");
	*szReturn = 0;
	CString szNumber;
	int nCount = m_Book.GetItemCount();
	for(int n = 0;n < nCount;n++)
	{
		szNumber = m_Book.GetItemText(n,1);
		if(!StrCmp(szNumber.GetBuffer(0),pszNumber))
		{
			StringCchCopy(szReturn,64,m_Book.GetItemText(n,0));
			break;
		}// if(!StrCmp(szNumber.GetBuffer(0),pszNumber))
	}// for(int n = 0;n < nCount;n++)

	return szReturn;
}
//****************************************************************************************
// Read data
void CBookDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_BOOK, m_Book);
}
//****************************************************************************************
// Create dialog
BOOL CBookDlg::OnInitDialog(VOID)
{
	CDialog::OnInitDialog();

	// Prepare phone book
	m_Book.SetExtendedStyle(LVS_EX_FULLROWSELECT | LVS_EX_GRIDLINES);
	m_Book.InsertColumn(0,_T("Name"),LVCFMT_LEFT,220);
	m_Book.InsertColumn(1,_T("Number"),LVCFMT_LEFT,180);

	LoadFromFile();
	
	return TRUE;
}
//****************************************************************************************
// Read phone book data from HD
VOID CBookDlg::LoadFromFile(VOID)
{
	CFile BookFile;
	CString szLine;
	TCHAR szName[64];
	TCHAR szNumber[64];
	int nPos;
	if(BookFile.Open(
		_T("Telefonbuch.txt"),
		CFile::modeRead 
		| CFile::shareDenyNone 
		| CFile::modeCreate
		| CFile::modeNoTruncate))
	{
		CArchive Reader(&BookFile,CArchive::load);
		while(Reader.ReadString(szLine))
		{
			_stscanf_s(szLine.GetBuffer(0),_T("%63[^;];%63s"),szName,63,szNumber,63);
			nPos = m_Book.InsertItem(0,szName);
			m_Book.SetItemText(nPos,1,szNumber);
		}
		BookFile.Close();
		SetCompareData();
	}// if(ListFile.Open...
}
//****************************************************************************************
// Save phone book data to HD
VOID CBookDlg::SaveToFile(VOID)
{
	CFile BookFile;
	if(BookFile.Open(
		_T("Telefonbuch.txt"),
		CFile::modeWrite 
		| CFile::shareDenyNone 
		| CFile::modeCreate))
	{
		int nCount = m_Book.GetItemCount();
		CString szName;
		CString szNumber;
		CString szLine;
		for(int n = 0;n < nCount;n++)
		{
			szName = m_Book.GetItemText(n,0);
			szNumber = m_Book.GetItemText(n,1);
			szLine = szName + ";" + szNumber + "\n";
			BookFile.Write(szLine,szLine.GetLength());
		}// for(int n = 0;n < nCount;n++)
		BookFile.Close();
	}// if(ListFile.Open...
}
//****************************************************************************************
// Insert compare data
VOID CBookDlg::SetCompareData(VOID)
{
	int nCount = m_Book.GetItemCount();
	for(int n = 0;n < nCount;n++)
	{
		m_Book.SetItemData(n,n);
	}// for(int n = 0;n < nCount,n++)
}
//****************************************************************************************
// Sort function
int CALLBACK CBookDlg::CompareFunc(LPARAM lParam1,LPARAM lParam2,LPARAM lParamSort)
{
	CString Text1 = m_pThis->m_Book.GetItemText((int)lParam1,(int)lParamSort);
	CString Text2 = m_pThis->m_Book.GetItemText((int)lParam2,(int)lParamSort);

	if(Text1 < Text2){return -1;}
	if(Text1 > Text2){return 1;}
	return 0;
}
//****************************************************************************************
// Evaluate variaous notifications
BOOL CBookDlg::OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult)
{
	// Nur Listenansicht bercksichtigen
	if(wParam == IDC_BOOK)
	{
		LPNMHDR pNmHdr = (LPNMHDR)lParam;
		switch(pNmHdr->code)
		{
		// Call by double click
		case NM_DBLCLK:
			{
				int nPos = m_Book.GetNextItem(-1,LVNI_ALL | LVNI_SELECTED);
				if(nPos >= 0)
				{
					CString szNumber = m_Book.GetItemText(nPos,1);
					CTI_TMakeCall(szNumber.GetBuffer(0));
				}// if(nPos >= 0)
				break;
			}
		// Column sort header
		case LVN_COLUMNCLICK:
			{
				LPNMLISTVIEW lpView = (LPNMLISTVIEW)lParam;
				m_Book.SortItems(CompareFunc,lpView->iSubItem);
				SetCompareData();
				return TRUE;
			}// case LVN_COLUMNCLICK
		// Press button
		case LVN_KEYDOWN:
			{
				// Insert
				LPNMKEY pNmKey = (LPNMKEY)lParam;
				if(LOWORD(pNmKey->nVKey) == VK_INSERT)
				{
					CInsertNameDlg InsertNameDlg(this);
					if(InsertNameDlg.DoModal() == IDOK)
					{
						CString szName = InsertNameDlg.GetName();
						CString szNumber = InsertNameDlg.GetNumber();
						int nPos = m_Book.InsertItem(0,szName);
						m_Book.SetItemText(nPos,1,szNumber);
						SetCompareData();
					}// if(InsertNameDlg.GetReturnCode() == IDOK)

				}// if(LOWORD(pNmKey->nVKey) == VK_INSERT)

				// Delete
				if(LOWORD(pNmKey->nVKey) == VK_DELETE)
				{
					int nPos = m_Book.GetNextItem(-1,LVNI_ALL | LVNI_SELECTED);
					if(nPos >= 0)
					{
						m_Book.DeleteItem(nPos);
						SetCompareData();
					}// if(nPos >= 0)
				}// if(LOWORD(pNmKey->nVKey) == VK_DELETE)
				break;
			}// case LVN_KEYDOWN
		}// switch(pNmHdr->code)
	}// if(wParam == IDC_BOOK)
	
	return CWnd::OnNotify(wParam,lParam,pResult);
}
//****************************************************************************************
BEGIN_MESSAGE_MAP(CBookDlg, CDialog)
	ON_WM_SHOWWINDOW()
	ON_WM_DESTROY()
END_MESSAGE_MAP()
//****************************************************************************************
// Show phone book
void CBookDlg::OnShowWindow(BOOL bShow, UINT nStatus)
{
	CDialog::OnShowWindow(bShow, nStatus);

	// Resize phone book
	if(bShow)
	{
		CRect Pos;
		GetWindowRect(&Pos);
		m_Book.MoveWindow(0,0,(Pos.right - Pos.left)-9,LIST_SIZE - 9);
	}// if(bShow)
}
//****************************************************************************************
// Remove dialog
void CBookDlg::OnDestroy()
{
	CDialog::OnDestroy();

	SaveToFile();
}
//****************************************************************************************
